@@ -1,20 +1,8 @@
-/***************************************************************************
- *   Copyright  2010 Johannes Loehnert <loehnert.kde@gmx.de>
- *
- *   This program is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU General Public
- *   License as published by the Free Software Foundation; either
- *   version 2 of the License, or (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- ***************************************************************************/
+/*
+    SPDX-FileCopyrightText: 2010 Johannes Loehnert <loehnert.kde@gmx.de>
+
+    SPDX-License-Identifier: GPL-2.0-or-later
+*/
 
 #include "grid.h"
 
@@ -43,7 +31,11 @@ QList<int> popIntLine(QList<QByteArray> &input) {
     QList<int> result;
     if (input.size() == 0) return result;
 
-    QStringList parts = QString(input.takeFirst()).split(' ', QString::SkipEmptyParts);
+#if QT_VERSION >= QT_VERSION_CHECK(5, 14, 0)
+    const QStringList parts = QString::fromUtf8(input.takeFirst()).split(QLatin1Char(' '), Qt::SkipEmptyParts);
+#else
+    QStringList parts = QString::fromUtf8(input.takeFirst()).split(QLatin1Char(' '), QString::SkipEmptyParts);
+#endif
     bool ok;
 
     for (int i=0; i<parts.size(); ++i) {
@@ -63,7 +55,11 @@ QList<qreal> popFloatLine(QList<QByteArray> &input) {
     QList<qreal> result;
     if (input.size() == 0) return result;
 
-    QStringList parts = QString(input.takeFirst()).split(' ', QString::SkipEmptyParts);
+#if QT_VERSION >= QT_VERSION_CHECK(5, 14, 0)
+    const QStringList parts = QString::fromUtf8(input.takeFirst()).split(QLatin1Char(' '), Qt::SkipEmptyParts);
+#else
+    QStringList parts = QString::fromUtf8(input.takeFirst()).split(QLatin1Char(' '), QString::SkipEmptyParts);
+#endif
     bool ok;
 
     for (int i=0; i<parts.size(); ++i) {
@@ -275,7 +271,7 @@ void IrregularMode::generateGrid(GoldbergEngine *e, int piece_count) const {
         }
         delete pfinder;
         pfinder = new_pfinder;
-        new_pfinder = NULL;
+        new_pfinder = nullptr;
     }
 
 
@@ -287,7 +283,7 @@ void IrregularMode::generateGrid(GoldbergEngine *e, int piece_count) const {
 bool IrregularMode::checkForQVoronoi() {
     QProcess process;
 
-    process.start(QStringLiteral("qvoronoi"));
+    process.start(QStringLiteral("qvoronoi"), QStringList());
     process.waitForStarted();
     if (process.error() == QProcess::FailedToStart) {
         return false;
@@ -481,7 +477,7 @@ void IrregularMode::generateVoronoiGrid(GoldbergEngine *e, QList<QPointF> cell_c
         }
         else {
             // invisible ridge: no border definition
-            p_plug = NULL;
+            p_plug = nullptr;
         }
 
         // add the border to the cells
