@@ -1,21 +1,9 @@
-/***************************************************************************
- *   Copyright 2009 Stefan Majewsky <majewsky@gmx.net>
- *   Copyright 2014 Ian Wadham <iandw.au@gmail.com>
- *
- *   This program is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU General Public
- *   License as published by the Free Software Foundation; either
- *   version 2 of the License, or (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
-***************************************************************************/
+/*
+    SPDX-FileCopyrightText: 2009 Stefan Majewsky <majewsky@gmx.net>
+    SPDX-FileCopyrightText: 2014 Ian Wadham <iandw.au@gmail.com>
+
+    SPDX-License-Identifier: GPL-2.0-or-later
+*/
 
 #include "gameplay.h"
 #include "palapeli_debug.h"
@@ -57,25 +45,25 @@ typedef QPair<int, int> DoubleIntPair;
 
 const int Palapeli::GamePlay::LargePuzzle = 300;
 
-const QString HeaderSaveGroup    ("-PalapeliSavedPuzzle");
-const QString HolderSaveGroup    ("Holders");
-const QString LocationSaveGroup  ("XYCo-ordinates");
-const QString FormerSaveGroup    ("SaveGame");
-const QString AppearanceSaveGroup("Appearance");
-const QString PreviewSaveGroup   ("PuzzlePreview");
+const QString HeaderSaveGroup     = QStringLiteral("-PalapeliSavedPuzzle");
+const QString HolderSaveGroup     = QStringLiteral("Holders");
+const QString LocationSaveGroup   = QStringLiteral("XYCo-ordinates");
+const QString FormerSaveGroup     = QStringLiteral("SaveGame");
+const QString AppearanceSaveGroup = QStringLiteral("Appearance");
+const QString PreviewSaveGroup    = QStringLiteral("PuzzlePreview");
 
 Palapeli::GamePlay::GamePlay(MainWindow* mainWindow)
 	: QObject(mainWindow)
 	, m_centralWidget(new QStackedWidget)
 	, m_collectionView(new Palapeli::CollectionView)
 	, m_puzzleTable(new Palapeli::PuzzleTableWidget)
-	, m_puzzlePreview(0)
+	, m_puzzlePreview(nullptr)
 	, m_mainWindow(mainWindow)
-	, m_puzzle(0)
+	, m_puzzle(nullptr)
 	, m_pieceAreaSize(QSizeF(32.0, 32.0))	// Allow 1024 pixels initially.
 	, m_savegameTimer(new QTimer(this))
-	, m_currentHolder(0)
-	, m_previousHolder(0)
+	, m_currentHolder(nullptr)
+	, m_previousHolder(nullptr)
 	, m_loadingPuzzle(false)
 	, m_restoredGame(false)
 	, m_originalPieceCount(0)
@@ -89,15 +77,13 @@ Palapeli::GamePlay::GamePlay(MainWindow* mainWindow)
 	m_viewList << m_puzzleTable->view();
 	m_savegameTimer->setInterval(500); //write savegame twice per second at most
 	m_savegameTimer->setSingleShot(true);
-	connect(m_savegameTimer, SIGNAL(timeout()), this, SLOT(updateSavedGame()));
-	connect(this, SIGNAL(reportProgress(int,int)),
-		m_puzzleTable, SLOT(reportProgress(int,int)));
-	connect(this, SIGNAL(victoryAnimationFinished()),
-		m_puzzleTable->view(), SLOT(startVictoryAnimation()));
-	connect(m_puzzleTable->view(),
-		SIGNAL(teleport(Piece*,QPointF,View*)),
-		this,
-		SLOT(teleport(Piece*,QPointF,View*)));
+	connect(m_savegameTimer, &QTimer::timeout, this, &GamePlay::updateSavedGame);
+	connect(this, &GamePlay::reportProgress,
+		m_puzzleTable, &PuzzleTableWidget::reportProgress);
+	connect(this, &GamePlay::victoryAnimationFinished,
+		m_puzzleTable->view(), &View::startVictoryAnimation);
+	connect(m_puzzleTable->view(), &View::teleport,
+		this, &GamePlay::teleport);
 }
 
 Palapeli::GamePlay::~GamePlay()
@@ -113,8 +99,8 @@ void Palapeli::GamePlay::deletePuzzleViews()
 		Palapeli::View* view = m_viewList.takeLast();
 		Palapeli::Scene* scene = view->scene();
 		qCDebug(PALAPELI_LOG) << "DISCONNECT SLOT(positionChanged(int))";
-		disconnect(scene, SIGNAL(saveMove(int)),
-			   this, SLOT(positionChanged(int)));
+		disconnect(scene, &Scene::saveMove,
+			   this, &GamePlay::positionChanged);
 		qCDebug(PALAPELI_LOG) << "scene->clearPieces();";
 		view->interactorManager()->resetActiveTriggers();
 		scene->clearPieces();
@@ -131,8 +117,8 @@ void Palapeli::GamePlay::deletePuzzleViews()
 void Palapeli::GamePlay::init()
 {
 	// Set up the collection view.
-	m_collectionView->setModel(Palapeli::Collection::instance());
-	connect(m_collectionView, SIGNAL(playRequest(Palapeli::Puzzle*)), SLOT(playPuzzle(Palapeli::Puzzle*)));
+	m_collectionView->setModel(Palapeli::Collection::instance(m_mainWindow));
+	connect(m_collectionView, &CollectionView::playRequest, this, &GamePlay::playPuzzle);
 
 	// Set up the puzzle table.
 	m_puzzleTable->showStatusBar(Settings::showStatusBar());
@@ -144,9 +130,9 @@ void Palapeli::GamePlay::init()
 	m_mainWindow->setCentralWidget(m_centralWidget);
 	// Get some current action states from the collection.
 	m_canDeletePuzzle = m_mainWindow->actionCollection()->
-				action("game_delete")->isEnabled();
+				action(QStringLiteral("game_delete"))->isEnabled();
 	m_canExportPuzzle = m_mainWindow->actionCollection()->
-				action("game_export")->isEnabled();
+				action(QStringLiteral("game_export"))->isEnabled();
 	// Enable collection actions and disable playing actions initially.
 	setPalapeliMode(false);
 }
@@ -171,14 +157,14 @@ void Palapeli::GamePlay::playPuzzle(Palapeli::Puzzle* puzzle)
 	t.start();	// IDW test. START the clock.
 	// we need to load the preview every time, although when the puzzle
 	// is already loaded because the preview is destroyed in actionGoCollection()
-	QTimer::singleShot(0, this, SLOT(loadPreview()));
+	QTimer::singleShot(0, this, &GamePlay::loadPreview);
 
 	qCDebug(PALAPELI_LOG) << "START playPuzzle(): elapsed 0";
 	// Get some current action states from the collection.
 	m_canDeletePuzzle = m_mainWindow->actionCollection()->
-				action("game_delete")->isEnabled();
+				action(QStringLiteral("game_delete"))->isEnabled();
 	m_canExportPuzzle = m_mainWindow->actionCollection()->
-				action("game_export")->isEnabled();
+				action(QStringLiteral("game_export"))->isEnabled();
 	m_centralWidget->setCurrentWidget(m_puzzleTable);
 	m_puzzlePreview = new Palapeli::PuzzlePreview(m_mainWindow);
 
@@ -237,10 +223,10 @@ void Palapeli::GamePlay::loadPreview()
 	}
 
 	m_puzzlePreview->setVisible(Settings::puzzlePreviewVisible());
-	connect (m_puzzlePreview, SIGNAL(closing()),
-		SLOT(actionTogglePreview()));	// Hide preview: do not delete.
+	connect (m_puzzlePreview, &PuzzlePreview::closing,
+		 this, &GamePlay::actionTogglePreview);	// Hide preview: do not delete.
 	// sync with mainWindow
-	m_mainWindow->actionCollection()->action("view_preview")->
+	m_mainWindow->actionCollection()->action(QStringLiteral("view_preview"))->
 		setChecked(Settings::puzzlePreviewVisible());
 }
 
@@ -258,7 +244,7 @@ void Palapeli::GamePlay::actionGoCollection()
 	m_puzzlePreview = nullptr;
 	m_mainWindow->setCaption(QString());
 	// IDW TODO - Disable piece-holder actions.
-	foreach (Palapeli::View* view, m_viewList) {
+	for (Palapeli::View* view : qAsConst(m_viewList)) {
 		if (view != m_puzzleTable->view()) {
 			view->hide();
 		}
@@ -272,7 +258,7 @@ void Palapeli::GamePlay::actionTogglePreview()
 	// This action is OK during puzzle loading.
 	if (m_puzzlePreview) {
 		m_puzzlePreview->toggleVisible();
-		m_mainWindow->actionCollection()->action("view_preview")->
+		m_mainWindow->actionCollection()->action(QStringLiteral("view_preview"))->
 			setChecked(Settings::puzzlePreviewVisible());
 		// remember state
 		updateSavedGame();
@@ -302,7 +288,7 @@ void Palapeli::GamePlay::actionDelete()
 	QModelIndexList indexes = m_collectionView->selectedIndexes();
 	//ask user for confirmation
 	QStringList puzzleNames;
-	foreach (const QModelIndex& index, indexes)
+	for (const QModelIndex& index : qAsConst(indexes))
 		puzzleNames << index.data(Qt::DisplayRole).toString();
 	const int result = KMessageBox::warningContinueCancelList(m_mainWindow, i18n("The following puzzles will be deleted. This action cannot be undone."), puzzleNames);
 	if (result != KMessageBox::Continue)
@@ -329,21 +315,21 @@ void Palapeli::GamePlay::actionImport()
 															QString(),
 															filter);
 	Palapeli::Collection* coll = Palapeli::Collection::instance();
-	foreach (const QString& path, paths)
+	for (const QString& path : paths)
 		coll->importPuzzle(path);
 }
 
 void Palapeli::GamePlay::actionExport()
 {
-	QModelIndexList indexes = m_collectionView->selectedIndexes();
+	const QModelIndexList indexes = m_collectionView->selectedIndexes();
 	Palapeli::Collection* coll = Palapeli::Collection::instance();
-	foreach (const QModelIndex& index, indexes)
+	for (const QModelIndex& index : indexes)
 	{
 		Palapeli::Puzzle* puzzle = coll->puzzleFromIndex(index);
 		if (!puzzle)
 			continue;
 		//get puzzle name (as an initial guess for the file name)
-		puzzle->get(Palapeli::PuzzleComponent::Metadata).waitForFinished();
+		puzzle->get(Palapeli::PuzzleComponent::Metadata);
 		const Palapeli::MetadataComponent* cmp = puzzle->component<Palapeli::MetadataComponent>();
 		if (!cmp)
 			continue;
@@ -368,7 +354,7 @@ void Palapeli::GamePlay::createHolder()
 	QString name = QInputDialog::getText(m_mainWindow,
 		i18n("Create a piece holder"),
 		i18n("Enter a short name (optional):"),
-		QLineEdit::Normal, QString(""), &OK);
+		QLineEdit::Normal, QString(), &OK);
 	if (! OK) {
 		return;		// If CANCELLED, do not create a piece holder.
 	}
@@ -376,14 +362,12 @@ void Palapeli::GamePlay::createHolder()
 	// Merges/moves in new holders add to the progress bar and are saved.
 	Palapeli::View* view = m_viewList.last();
 	view->setCloseUp(true);	// New holders start in close-up scale.
-	connect(view->scene(), SIGNAL(saveMove(int)),
-		this, SLOT(positionChanged(int)));
-	connect(view,
-		SIGNAL(teleport(Piece*,QPointF,View*)),
-		this,
-		SLOT(teleport(Piece*,QPointF,View*)));
-	connect(view, SIGNAL(newPieceSelectionSeen(View*)),
-		this, SLOT(handleNewPieceSelection(View*)));
+	connect(view->scene(), &Scene::saveMove,
+		this, &GamePlay::positionChanged);
+	connect(view, &View::teleport,
+		this, &GamePlay::teleport);
+	connect(view, &View::newPieceSelectionSeen,
+		this, &GamePlay::handleNewPieceSelection);
 }
 
 void Palapeli::GamePlay::createHolder(const QString& name, bool sel)
@@ -392,10 +376,10 @@ void Palapeli::GamePlay::createHolder(const QString& name, bool sel)
 		new Palapeli::PieceHolder(m_mainWindow, m_pieceAreaSize, name);
 	m_viewList << h;
 	h->initializeZooming();			// Min. view 2x2 to 6x6 pieces.
-	connect(h, SIGNAL(selected(PieceHolder*)),
-		this, SLOT(changeSelectedHolder(PieceHolder*)));
-	connect (h, SIGNAL(closing(PieceHolder*)),
-		SLOT(closeHolder(PieceHolder*)));
+	connect(h, &PieceHolder::selected,
+		this, &GamePlay::changeSelectedHolder);
+	connect (h, &PieceHolder::closing,
+		this, &GamePlay::closeHolder);
 	if (sel) {
 		changeSelectedHolder(h);
 	}
@@ -443,10 +427,10 @@ void Palapeli::GamePlay::selectAll()
 {
 	qCDebug(PALAPELI_LOG) << "GamePlay::selectAll() entered";
 	if (m_currentHolder) {
-		QList<Palapeli::Piece*> pieces =
+		const QList<Palapeli::Piece*> pieces =
 					m_currentHolder->scene()->pieces();
 		if (! pieces.isEmpty()) {
-			foreach (Palapeli::Piece* piece, pieces) {
+			for (Palapeli::Piece* piece : pieces) {
 				piece->setSelected(true);
 			}
 			handleNewPieceSelection(m_currentHolder);
@@ -486,7 +470,7 @@ void Palapeli::GamePlay::rearrangePieces()
 		return;
 	}
 	QRectF bRect;
-	foreach (Palapeli::Piece* piece, selectedPieces) {
+	for (Palapeli::Piece* piece : qAsConst(selectedPieces)) {
 		bRect |= piece->sceneBareBoundingRect();
 	}
 	Palapeli::Scene* scene = view->scene();
@@ -494,7 +478,7 @@ void Palapeli::GamePlay::rearrangePieces()
 	scene->initializeGrid(((view == m_currentHolder) &&
 			(selectedPieces.count() == scene->pieces().count())) ?
 				QPointF(0.0, 0.0) : bRect.topLeft());
-	foreach (Palapeli::Piece* piece, selectedPieces) {
+	for (Palapeli::Piece* piece : qAsConst(selectedPieces)) {
 		scene->addToGrid(piece);
 	}
 	if (view == m_currentHolder) {
@@ -555,7 +539,7 @@ void Palapeli::GamePlay::teleport(Palapeli::Piece* pieceUnderMouse,
 		selectedPieces = getSelectedPieces(view);
                 if (!selectedPieces.isEmpty()) {
 			// Transfer from the puzzle table to a piece-holder.
-			foreach (Palapeli::Piece* piece, selectedPieces) {
+			for (Palapeli::Piece* piece : qAsConst(selectedPieces)) {
 				if (piece->representedAtomicPieces().count()
 					> 6) {
 					int ans = 0;
@@ -623,8 +607,8 @@ void Palapeli::GamePlay::handleNewPieceSelection(Palapeli::View* view)
 	// De-select pieces on puzzle table, to prevent teleport bounce-back.
 	Palapeli::View* m_puzzleTableView = m_puzzleTable->view();
 	if (view != m_puzzleTableView) {	// Pieces selected in a holder.
-		foreach (Palapeli::Piece* piece,
-				getSelectedPieces(m_puzzleTableView)) {
+		const auto selectedPieces = getSelectedPieces(m_puzzleTableView);
+		for (Palapeli::Piece* piece : selectedPieces) {
 			piece->setSelected(false);
 		}
 	}
@@ -647,21 +631,23 @@ void Palapeli::GamePlay::transferPieces(const QList<Palapeli::Piece*> &pieces,
 		m_puzzleTableScene->initializeGrid(scenePos);
 	}
 	Palapeli::Scene* scene = dest->scene();
-	foreach (Palapeli::Piece* piece, scene->pieces()) {
+	const auto sccenePieces = scene->pieces();
+	for (Palapeli::Piece* piece : sccenePieces) {
 		// Clear all previous selections in the destination scene.
 		if (piece->isSelected()) {
 			piece->setSelected(false);
 		}
 	}
-	foreach (Palapeli::Piece* piece, pieces) {
+	for (Palapeli::Piece* piece : pieces) {
 		// Leave the new arrivals selected, connected and in a grid.
 		scene->addPieceToList(piece);
 		scene->addItem(piece);
 		scene->addToGrid(piece);
 		piece->setSelected(true);
-		connect(piece, SIGNAL(moved(bool)),
-			scene, SLOT(pieceMoved(bool)));
+		connect(piece, &Piece::moved,
+			scene, &Scene::pieceMoved);
 	}
+	source->scene()->update();
 	scene->setSceneRect(scene->extPiecesBoundingRect());
 	if (! destIsPuzzleTable) {
 		dest->centerOn(pieces.last()->sceneBareBoundingRect().center());
@@ -676,31 +662,41 @@ void Palapeli::GamePlay::setPalapeliMode(bool playing)
 	// puzzle, both sets of actions are disabled, because they cannot work
 	// concurrently with loading (enPlaying and enCollection both false).
 
-	const char* playingActions[] = {"view_collection", "game_restart",
-					"view_preview", "move_create_holder",
-					"move_delete_holder", "move_select_all",
-					"move_rearrange", "view_zoom_in",
-					"view_zoom_out", "END" };
-	const char* collectionActions[] = {"game_new", "game_delete",
-					"game_import", "game_export", "END" };
+	const QString playingActions[] = {
+            QStringLiteral("view_collection"),
+            QStringLiteral("game_restart"),
+            QStringLiteral("view_preview"),
+            QStringLiteral("move_create_holder"),
+            QStringLiteral("move_delete_holder"),
+            QStringLiteral("move_select_all"),
+            QStringLiteral("move_rearrange"),
+            QStringLiteral("view_zoom_in"),
+            QStringLiteral("view_zoom_out"),
+        };
+	const QString collectionActions[] = {
+            QStringLiteral("game_new"),
+            QStringLiteral("game_delete"),
+            QStringLiteral("game_import"),
+            QStringLiteral("game_export"),
+        };
 	bool enPlaying    = (! m_loadingPuzzle) && playing;
 	bool enCollection = (! m_loadingPuzzle) && (! playing);
 
-	for (uint i = 0; (strcmp (playingActions[i], "END") != 0); i++) {
+	for (const auto &actionId : playingActions) {
 		m_mainWindow->actionCollection()->
-			action(playingActions[i])->setEnabled(enPlaying);
+			action(actionId)->setEnabled(enPlaying);
 	}
-	for (uint i = 0; (strcmp (collectionActions[i], "END") != 0); i++) {
+	for (const auto &actionId : collectionActions) {
 		m_mainWindow->actionCollection()->
-			action(collectionActions[i])->setEnabled(enCollection);
+			action(actionId)->setEnabled(enCollection);
 	}
 	// The collection view may enable or disable Delete and Export actions,
 	// depending on what puzzle, if any, is currently selected.
 	if (enCollection) {
 		m_mainWindow->actionCollection()->
-			action("game_delete")->setEnabled(m_canDeletePuzzle);
+			action(QStringLiteral("game_delete"))->setEnabled(m_canDeletePuzzle);
 		m_mainWindow->actionCollection()->
-			action("game_export")->setEnabled(m_canExportPuzzle);
+			action(QStringLiteral("game_export"))->setEnabled(m_canExportPuzzle);
 	}
 	m_playing = playing;
 }
@@ -710,7 +706,7 @@ QList<Palapeli::Piece*> Palapeli::GamePlay::getSelectedPieces(Palapeli::View* v)
 	qCDebug(PALAPELI_LOG) << "ENTERED GamePlay::getSelectedPieces(): PuzzleTable" << (v == m_puzzleTable->view());
 	const QList<QGraphicsItem*> sel = v->scene()->selectedItems();
 	QList<Palapeli::Piece*> pieces;
-	foreach (QGraphicsItem* item, sel) {
+	for (QGraphicsItem* item : sel) {
 		Palapeli::Piece* p = Palapeli::Piece::fromSelectedItem(item);
 		if (p) {
 			pieces << p;
@@ -740,7 +736,7 @@ void Palapeli::GamePlay::loadPuzzle()
 	setPalapeliMode(false);
 	// Stop autosaving and progress-reporting and start the loading-widget.
 	m_savegameTimer->stop(); // Just in case it is running ...
-	emit reportProgress(0, 0);
+	Q_EMIT reportProgress(0, 0);
 	// Is there a saved game?
 	const QString puzzleLoc(
 				QStandardPaths::locate(QStandardPaths::AppLocalDataLocation,
@@ -755,7 +751,7 @@ void Palapeli::GamePlay::loadPuzzle()
 		}
 	}
 	// Return to the event queue to start the loading-widget graphics ASAP.
-	QTimer::singleShot(0, this, SLOT(loadPuzzleFile()));
+	QTimer::singleShot(0, this, &GamePlay::loadPuzzleFile);
 	qCDebug(PALAPELI_LOG) << "END loadPuzzle()";
 }
 
@@ -772,12 +768,8 @@ void Palapeli::GamePlay::loadPuzzleFile()
 	// It is loaded asynchronously and processed one piece at a time.
 	m_loadedPieces.clear();
 	if (m_puzzle) {
-		Palapeli::FutureWatcher* watcher = new Palapeli::FutureWatcher;
-		connect(watcher, SIGNAL(finished()), SLOT(loadNextPiece()));
-		connect(watcher, SIGNAL(finished()),
-			watcher, SLOT(deleteLater()));
-		watcher->setFuture(
-			m_puzzle->get(Palapeli::PuzzleComponent::Contents));
+		m_puzzle->get(Palapeli::PuzzleComponent::Contents);
+		QTimer::singleShot(0, this, &Palapeli::GamePlay::loadNextPiece);
 	}
 	qCDebug(PALAPELI_LOG) << "Finish loadPuzzleFile(): time" << t.restart();
 }
@@ -817,9 +809,9 @@ void Palapeli::GamePlay::loadNextPiece()
 
 		// Continue with next piece or next stage, after event loop run.
 		if (contents.pieces.size() > m_loadedPieces.size())
-			QTimer::singleShot(0, this, SLOT(loadNextPiece()));
+			QTimer::singleShot(0, this, &Palapeli::GamePlay::loadNextPiece);
 		else
-			QTimer::singleShot(0, this, SLOT(loadPiecePositions()));
+			QTimer::singleShot(0, this, &Palapeli::GamePlay::loadPiecePositions);
 		return;
 	}
 }
@@ -833,11 +825,11 @@ void Palapeli::GamePlay::loadPiecePositions()
 	m_originalPieceCount = m_loadedPieces.count();
 	const Palapeli::PuzzleContents contents = m_puzzle->component<Palapeli::ContentsComponent>()->contents;
 	//add piece relations
-	foreach (const DoubleIntPair& relation, contents.relations) {
+	for (const DoubleIntPair& relation : contents.relations) {
 		Palapeli::Piece* firstPiece =
-				m_loadedPieces.value(relation.first, 0);
+				m_loadedPieces.value(relation.first, nullptr);
 		Palapeli::Piece* secondPiece =
-				m_loadedPieces.value(relation.second, 0);
+				m_loadedPieces.value(relation.second, nullptr);
 		firstPiece->addLogicalNeighbors(QList<Palapeli::Piece*>()
 				<< secondPiece);
 		secondPiece->addLogicalNeighbors(QList<Palapeli::Piece*>()
@@ -885,12 +877,12 @@ void Palapeli::GamePlay::loadPiecePositions()
 		m_currentHolder = nullptr;
 		for (int groupID = 1; groupID <= nHolders; groupID++) {
 			KConfigGroup holder (&savedConfig,
-					QString("Holder_%1").arg(groupID));
+					QStringLiteral("Holder_%1").arg(groupID));
 			// Re-create a piece-holder and add it to m_viewList.
 			qCDebug(PALAPELI_LOG) << "RE-CREATE HOLDER"
-				 << QString("Holder_%1").arg(groupID) << "name"
-				 << holder.readEntry("Name", QString(""));
-			createHolder(holder.readEntry("Name", QString("")),
+				 << QStringLiteral("Holder_%1").arg(groupID) << "name"
+				 << holder.readEntry("Name", QString());
+			createHolder(holder.readEntry("Name", QString()),
 				     holder.readEntry("Selected", false));
 			// Restore the piece-holder's size and position.
 			QRect r = holder.readEntry("Geometry", QRect());
@@ -929,7 +921,7 @@ void Palapeli::GamePlay::loadPiecePositions()
 		}
 		qCDebug(PALAPELI_LOG) << "FINISHED POSITIONING PIECES";
 		// Each scene re-merges pieces, as required, with no animation.
-		foreach (Palapeli::View* view, m_viewList) {
+		for (Palapeli::View* view : qAsConst(m_viewList)) {
 			view->scene()->mergeLoadedPieces();
 		}
 	}
@@ -953,7 +945,7 @@ void Palapeli::GamePlay::loadPiecePositions()
 
 		// Find the size of the area required for the solution.
 		QRectF r;
-		foreach (Palapeli::Piece* piece, piecePool) {
+		for (Palapeli::Piece* piece : qAsConst(piecePool)) {
 			r |= piece->sceneBareBoundingRect();
 		}
 		int xResv = 0;
@@ -1040,7 +1032,7 @@ void Palapeli::GamePlay::loadPiecePositions()
 	qreal handleWidth = qMin(s.width(), s.height())/100.0;
 	m_puzzleTableScene->addMargin(handleWidth, 0.5*handleWidth);
 	// Add all the pieces to the puzzle table and piece-holder scenes.
-	foreach (Palapeli::View* view, m_viewList) {
+	for (Palapeli::View* view : qAsConst(m_viewList)) {
 		Palapeli::Scene* scene = view->scene();
 		scene->addPieceItemsToScene();
 		if (scene != m_puzzleTableScene) {
@@ -1059,7 +1051,7 @@ void Palapeli::GamePlay::finishLoading()
 	// Start each scene and view.
 	qCDebug(PALAPELI_LOG) << "COUNTING CURRENT PIECES";
 	m_currentPieceCount = 0;
-	foreach (Palapeli::View* view, m_viewList) {
+	for (Palapeli::View* view : qAsConst(m_viewList)) {
 		Palapeli::Scene* scene = view->scene();
 		m_currentPieceCount = m_currentPieceCount +
 					scene->pieces().size();
@@ -1073,7 +1065,7 @@ void Palapeli::GamePlay::finishLoading()
 		}
 	}
 	// Initialize external progress display, hide loading widget, show view.
-	emit reportProgress(m_originalPieceCount, m_currentPieceCount);
+	Q_EMIT reportProgress(m_originalPieceCount, m_currentPieceCount);
 	// Adjust zoom-levels, center the view, show autosave message if needed.
 	m_puzzleTable->view()->puzzleStarted();
 	if (!m_restoredGame && (m_originalPieceCount >= LargePuzzle)) {
@@ -1144,16 +1136,14 @@ void Palapeli::GamePlay::finishLoading()
 		}
 	}
 	// Connect moves and merges of pieces to autosaving and progress-report.
-	foreach (Palapeli::View* view, m_viewList) {
-		connect(view->scene(), SIGNAL(saveMove(int)),
-			this, SLOT(positionChanged(int)));
+	for (Palapeli::View* view : qAsConst(m_viewList)) {
+		connect(view->scene(), &Scene::saveMove,
+			this, &GamePlay::positionChanged);
 		if (view != m_puzzleTable->view()) {
-			connect(view,
-				SIGNAL(teleport(Piece*,QPointF,View*)),
-				this,
-				SLOT(teleport(Piece*,QPointF,View*)));
-			connect(view, SIGNAL(newPieceSelectionSeen(View*)),
-				this, SLOT(handleNewPieceSelection(View*)));
+			connect(view, &View::teleport,
+				this, &GamePlay::teleport);
+			connect(view, &View::newPieceSelectionSeen,
+				this, &GamePlay::handleNewPieceSelection);
 		}
 	}
 	// Enable playing actions.
@@ -1165,7 +1155,7 @@ void Palapeli::GamePlay::finishLoading()
 void Palapeli::GamePlay::calculatePieceAreaSize()
 {
 	m_pieceAreaSize = QSizeF(0.0, 0.0);
-	foreach (Palapeli::Piece* piece, m_loadedPieces) {
+	for (Palapeli::Piece* piece : qAsConst(m_loadedPieces)) {
 		m_pieceAreaSize = m_pieceAreaSize.expandedTo
 				(piece->sceneBareBoundingRect().size());
 	}
@@ -1180,17 +1170,17 @@ void Palapeli::GamePlay::playVictoryAnimation()
 	animation->setStartValue(m_puzzleTableScene->sceneRect());
 	animation->setEndValue(m_puzzleTableScene->extPiecesBoundingRect());
 	animation->setDuration(1000);
-	connect(animation, SIGNAL(finished()),
-		this, SLOT(playVictoryAnimation2()));
+	connect(animation, &QAbstractAnimation::finished,
+		this, &GamePlay::playVictoryAnimation2);
 	animation->start(QAbstractAnimation::DeleteWhenStopped);
 }
 
 void Palapeli::GamePlay::playVictoryAnimation2()
 {
 	m_puzzleTableScene->setSceneRect(m_puzzleTableScene->extPiecesBoundingRect());
-	QTimer::singleShot(100, this, SIGNAL(victoryAnimationFinished()));
+	QTimer::singleShot(100, this, &GamePlay::victoryAnimationFinished);
 	// Give the View some time to play its part of the victory animation.
-	QTimer::singleShot(1500, this, SLOT(playVictoryAnimation3()));
+	QTimer::singleShot(1500, this, &GamePlay::playVictoryAnimation3);
 }
 
 void Palapeli::GamePlay::playVictoryAnimation3()
@@ -1205,7 +1195,7 @@ void Palapeli::GamePlay::positionChanged(int reduction)
 		bool victory = (m_currentPieceCount > 1) &&
 			       ((m_currentPieceCount - reduction) <= 1);
 		m_currentPieceCount = m_currentPieceCount - reduction;
-		emit reportProgress(m_originalPieceCount, m_currentPieceCount);
+		Q_EMIT reportProgress(m_originalPieceCount, m_currentPieceCount);
 		if (victory) {
 			playVictoryAnimation();
 		}
@@ -1234,11 +1224,11 @@ void Palapeli::GamePlay::updateSavedGame()
 	headerGroup.writeEntry("N_Holders", m_viewList.count() - 1);
 
 	int groupID = 0;
-	foreach (Palapeli::View* view, m_viewList) {
+	for (Palapeli::View* view : qAsConst(m_viewList)) {
 	    bool isHolder = (view != m_puzzleTable->view());
 	    if (isHolder) {
 		KConfigGroup holderDetails(&savedConfig,
-			QString("Holder_%1").arg(groupID));
+			QStringLiteral("Holder_%1").arg(groupID));
 		Palapeli::PieceHolder* holder =
 			qobject_cast<Palapeli::PieceHolder*>(view);
 		bool selected = (view == m_currentHolder);
@@ -1248,9 +1238,10 @@ void Palapeli::GamePlay::updateSavedGame()
 			QRect(view->frameGeometry().topLeft(), view->size()));
 	    }
 	    const QList<Palapeli::Piece*> pieces = view->scene()->pieces();
-	    foreach (Palapeli::Piece* piece, pieces) {
+	    for (Palapeli::Piece* piece : pieces) {
 		const QPointF position = piece->pos();
-		foreach (int atomicPieceID, piece->representedAtomicPieces()) {
+		const auto atomicPieces = piece->representedAtomicPieces();
+		for (int atomicPieceID : atomicPieces) {
 		    const QString ID = QString::number(atomicPieceID);
 		    locationGroup.writeEntry(ID, position);
 		    if (isHolder) {
@@ -1314,7 +1305,7 @@ void Palapeli::GamePlay::restorePuzzleSettings(KConfig* savedConfig)
 	// Ask TextureHelper to re-draw background (but only after KConfigDialog
 	// has written the settings, which might happen after this slot call).
 	QTimer::singleShot(0, Palapeli::TextureHelper::instance(),
-				SLOT(readSettings()));
+				&Palapeli::TextureHelper::readSettings);
 
 	if (savedConfig->hasGroup(PreviewSaveGroup)) {
 		KConfigGroup previewGroup(savedConfig, PreviewSaveGroup);
